// parameter class definitions
// automatically generated by make-parameter-class
// part of libecsb - the executable cell systems biology library
// copyright 2008 jamie twycross, jpt AT cs.nott.ac.uk
// released under GNU GPL version 3

#include "parameters.h"

#include <string.h>
#include <stdlib.h>

using namespace ECSB;

// constructor
MCParameters::MCParameters(void) 
{
    // set initial values
    this->model_checker = "PRISM";
    this->PRISM_model = "PRISM_model.sm";
    this->parameters_file = "modelParameters.xml";
    this->states_file = "states.psm";
    this->transitions_file = "transitions.psm";
    this->results_file = "results.psm";
}

//read parameters from a file
void MCParameters::read(std::string xmlFile)
{
    //Use of the DOM parser
    DomParser parser;

    //Parse our XML file.
    parser.parse_file(xmlFile);

    if(!parser)
    {
      cerr << "Parsing Unsuccesfull" << std::endl;
      exit(EXIT_FAILURE);
    }

    //Get the root node of the parameters
    xmlpp::Node* parametersDeclaration = parser.get_document()->get_root_node();

    //We get the name of the model
    xmlpp::Element* model = dynamic_cast<xmlpp::Element*>(parametersDeclaration);

    //Get the declaration of the different parameters
    xmlpp::Node::NodeList component = parametersDeclaration->get_children("parameterSet");
    xmlpp::Node* parameterSet = component.front();
    xmlpp::Node::NodeList parameters = parameterSet->get_children("parameter");

    for(xmlpp::Node::NodeList::iterator par = parameters.begin(); par != parameters.end(); ++par)
    {
        xmlpp::Element* parElement = dynamic_cast<xmlpp::Element*>(*par);
        std::string parameterName = (parElement->get_attribute("name"))->get_value();

        if(parameterName == "P_system_specification")
        {
            this->P_system_specification = (parElement->get_attribute("value"))->get_value();
        }
        else if (parameterName == "model_checker")
        {
            this->model_checker = (parElement->get_attribute("value"))->get_value();
        }
        else if (parameterName == "PRISM_model")
        {
            this->PRISM_model = (parElement->get_attribute("value"))->get_value();
        }
        else if (parameterName == "simulations_generated")
        {
            this->simulations_generated = (parElement->get_attribute("value"))->get_value();
        }
        else if (parameterName == "simulations_file_hdf5")
        {
            this->simulations_file_hdf5 = (parElement->get_attribute("value"))->get_value();
        }
        else if (parameterName == "mcss_params_file")
        {
            this->mcss_params_file = (parElement->get_attribute("value"))->get_value();
        }
        else if (parameterName == "simulations_file_MC2")
        {
            this->simulations_file_MC2 = (parElement->get_attribute("value"))->get_value();
        }
        else if (parameterName == "parameters_file")
        {
            this->parameters_file = (parElement->get_attribute("value"))->get_value();
        }
        else if (parameterName == "confidence")
        {
            this->confidence = fromStringTo<double>((parElement->get_attribute("value"))->get_value());
        }
        else if (parameterName == "formula_parameters")
        {
            this->formula_parameters = (parElement->get_attribute("value"))->get_value();
        }
        else if (parameterName == "model_parameters")
        {
            this->model_parameters = (parElement->get_attribute("value"))->get_value();
        }
        else if (parameterName == "number_samples")
        {
            this->number_samples = fromStringTo<unsigned long>((parElement->get_attribute("value"))->get_value());
        }
        else if (parameterName == "precision")
        {
            this->precision = fromStringTo<double>((parElement->get_attribute("value"))->get_value());
        }
        else if (parameterName == "results_file")
        {
            this->results_file = (parElement->get_attribute("value"))->get_value();
        }
        else if (parameterName == "states_file")
        {
            this->states_file = (parElement->get_attribute("value"))->get_value();
        }
        else if (parameterName == "task")
        {
            this->task = (parElement->get_attribute("value"))->get_value();
        }
        else if (parameterName == "temporal_formulas")
        {
            this->temporal_formulas = (parElement->get_attribute("value"))->get_value();
        }
        else if (parameterName == "transitions_file")
        {
            this->transitions_file = (parElement->get_attribute("value"))->get_value();
        }
        else
        {
            cerr << "Unknown parameter " << (*par)->get_name() <<std::endl;
            exit(EXIT_FAILURE);
        }

    }
}

