/*
 * Date (yymmdd):               090320
 * Version:                     0.2
 * Written by:                  Dr. Francisco J. Romero-Campero
 * Additional contributions by: Jonathan Blakes
 * (in no particular order)     Dr. Hongqing Cao
 *                              Dr. Natalio Krasnogor
 *				Dr. Jamie Twycross
 * Funded by:                   EPSRC grant EP/E017215/1
 * Website:                     http://www.infobiotic.org/
 * Contact:                     nxk@cs.nott.ac.uk
 */

#include <ecsb/ecsb.h>

#include "prismcommand.h"

using namespace std;
using namespace ECSB;
using namespace PMODELCHECKER;

PrismCommand::PrismCommand()
{

}

PrismCommand::PrismCommand(const PrismCommand& orig)
{
    this->identifier = orig.identifier;
    this->predicate = orig.predicate;
    this->rate = orig.rate;
    this->updates = orig.updates;
}


PrismCommand::PrismCommand(int option, std::string moduleName, ECSB::CAPRule rule,
        std::vector<int> position, std::vector<ECSB::automaton> automata)
{
    std::string ruleIdentifier = rule.getName() + "_" + moduleName;
    this->identifier = ruleIdentifier;

    createPredicate(option,moduleName,rule,position,automata);

    createRate(option,moduleName,rule);

    createUpdates(option,moduleName,rule,position,automata);

}

PrismCommand::~PrismCommand() {
}



void PrismCommand::createPredicate(int option, std::string moduleName, ECSB::CAPRule rule,
        std::vector<int> position, std::vector<ECSB::automaton> automata)
{
    if(option == 0)
    {
        std::vector<std::string> lhsInside = rule.getLhsInside();
        for(std::vector<std::string>::iterator obj = lhsInside.begin();
            obj != lhsInside.end();
            ++obj)
        {
            this->predicate += *obj + "_" + moduleName + " > ";
            this->predicate += somethingToString<int>(multiplicity(*obj, lhsInside) - 1);
            if(obj != lhsInside.end() -1)
                this->predicate += " & ";
        }


        std::vector<std::string> rhsInside = rule.getRhsInside();

        if(!rhsInside.empty())
            this->predicate += " & ";

        for(std::vector<std::string>::iterator obj = rhsInside.begin();
            obj != rhsInside.end();
            ++obj)
        {
            
            int effect = multiplicity(*obj,rhsInside) - multiplicity(*obj,lhsInside);
            
            if(effect != 0)
            {
                this->predicate += *obj + "_" + moduleName + " < ";
                this->predicate += "ub_" + *obj + "_" + moduleName;

                if(multiplicity(*obj, rhsInside) - 1 > 0)
                {
                    this->predicate += " - " + somethingToString<int>(multiplicity(*obj, rhsInside) - 1);
                }

                if(obj != rhsInside.end() - 1)
                    this->predicate += " & ";
            }
            
        }

    }
    else if(option == 1)
    {
        
        std::vector<std::string> lhsTranslocation = rule.getLhsTranslocation();
        for(std::vector<std::string>::iterator obj = lhsTranslocation.begin();
            obj != lhsTranslocation.end();
            ++obj)
        {
            this->predicate += *obj + "_" + moduleName + " > ";
            this->predicate += somethingToString<int>(multiplicity(*obj, lhsTranslocation) - 1);
            if(obj != lhsTranslocation.end() -1)
                this->predicate += " & ";
        }
    }
    else if(option == 2)
    {
        std::vector<std::string> rhsTranslocation = rule.getRhsTranslocation();
        for(std::vector<std::string>::iterator obj = rhsTranslocation.begin();
            obj != rhsTranslocation.end();
            ++obj)
        {

            std::string target = targetModuleName(rule,position,automata);
            this->predicate += *obj + "_" + target + " < ";
            this->predicate += "ub_" + *obj + "_" + target;

            if(multiplicity(*obj, rhsTranslocation) - 1 > 0)
            {
                this->predicate += " - " + somethingToString<int>(multiplicity(*obj, rhsTranslocation) - 1);
            }

            if(obj != rhsTranslocation.end() - 1)
                this->predicate += " & ";
        }
    }
    

}

void PrismCommand::createRate(int option, std::string moduleName, ECSB::CAPRule rule)
{

    if(option == 0)
    {
        this->rate = "c_" + this->identifier;//somethingToString<double>(rule.getConstant());

        std::vector<std::string> lhsInside = rule.getLhsInside();
        for(std::vector<std::string>::iterator obj = lhsInside.begin();
            obj != lhsInside.end();
            ++obj)
        {
            this->rate += " * " + *obj + "_" + moduleName;
        }

    }
    else if(option ==1)
    {
        this->rate = "c_" + this->identifier;//somethingToString<double>(rule.getConstant());

        std::vector<std::string> lhsTranslocation = rule.getLhsTranslocation();
        for(std::vector<std::string>::iterator obj = lhsTranslocation.begin();
            obj != lhsTranslocation.end();
            ++obj)
        {
            this->rate += " * " + *obj + "_" + moduleName;
        }

    }
    else if (option == 2)
    {
        this->rate = "1";
    }
}

void PrismCommand::createUpdates(int option, std::string moduleName, ECSB::CAPRule rule,std::vector<int> position, std::vector<ECSB::automaton> automata)
{
    if(option == 0)
    {
        std::vector<std::string> lhsInside = rule.getLhsInside();
        std::vector<std::string> rhsInside = rule.getRhsInside();

        for(std::vector<std::string>::iterator obj = lhsInside.begin();
            obj != lhsInside.end();
            ++obj)
        {
            int effect = multiplicity(*obj, rhsInside) - multiplicity(*obj, lhsInside);

            if(effect != 0)
            {
                std::string update;
                update += *obj + "_" + moduleName + "' = ";
                update += *obj + "_" + moduleName;
                update += somethingToString<int>(effect);
                (this->updates).push_back(update);
            }
        }

        for(std::vector<std::string>::iterator obj = rhsInside.begin();
            obj != rhsInside.end();
            ++obj)
        {
            int effect = multiplicity(*obj, rhsInside) - multiplicity(*obj, lhsInside);

            if(effect != 0)
            {
                std::string update;
                update += *obj + "_" + moduleName + "' = ";
                update += *obj + "_" + moduleName + " + ";
                update += somethingToString<int>(effect);
                (this->updates).push_back(update);
            }
        }

    }
    else if (option == 1)
    {
        std::vector<std::string> lhsTranslocation = rule.getLhsTranslocation();

        for(std::vector<std::string>::iterator obj = lhsTranslocation.begin();
            obj != lhsTranslocation.end();
            ++obj)
        {
                std::string update;
                update += *obj + "_" + moduleName + "' = ";
                update += *obj + "_" + moduleName + "-";
                update += somethingToString<int>(multiplicity(*obj, lhsTranslocation));

                (this->updates).push_back(update);
        }
    }
    else if (option == 2)
    {
        std::vector<std::string> rhsTranslocation = rule.getRhsTranslocation();

        for(std::vector<std::string>::iterator obj = rhsTranslocation.begin();
            obj != rhsTranslocation.end();
            ++obj)
        {
            std::string target = targetModuleName(rule,position,automata);
            std::string update;
            update += *obj + "_" + target + "' = ";
            update += *obj + "_" + target + " + ";
            update += somethingToString<int>(multiplicity(*obj,rhsTranslocation));
            (this->updates).push_back(update);
        }

    }

}

//Getters
std::string PrismCommand::getIdentifier()
{
    return this->identifier;
}

std::string PrismCommand::getPredicate()
{
    return this->predicate;
}

std::string PrismCommand::getRate()
{
    return this->rate;
}

std::vector<std::string> PrismCommand::getUpdates()
{
    return this->updates;
}

//This function computes the multiplicity of an object in a vector representing
//a multiset
int PrismCommand::multiplicity(string object, std::vector<std::string> multiset)
{
    int res = 0;

    for(vector<string>::iterator it = multiset.begin();
        it != multiset.end();
        ++it)
    {
	if(*it == object)
            ++res;
    }

    return res;
}


//Auxiliar to find target module compartment for inserting the commands
std::string PrismCommand::targetModuleName(ECSB::CAPRule rule,
        std::vector<int> position,
        std::vector<ECSB::automaton> automata)
{
    //The position of the target automaton is computed by adding the rule vector
    //to the position of the current automaton.
    std::vector<int> targetPosition;
    std::vector<int> ruleVector = rule.getVect();

    int dim = position.size();
    for(int i = 0; i < dim; ++i)
        targetPosition.push_back(ruleVector[i] + position[i]);

    //Finding the target automaton
    for(std::vector<ECSB::automaton>::iterator aut = automata.begin();
        aut != automata.end();
        ++aut)
    {
        std::vector<int> candidatePosition = aut->getPosition();
        if(candidatePosition == targetPosition)
        {
            //Constructing the targetModuleName
            std::string moduleName;

            if(position.size() == 1)
                moduleName += somethingToString<int>(targetPosition[0]);
            else if (targetPosition.size() == 2)
                moduleName += somethingToString<int>(targetPosition[0]) + "_" + somethingToString<int>(targetPosition[1]);
            else if (targetPosition.size() == 3)
            {
                moduleName += somethingToString<int>(targetPosition[0]) + "_" + somethingToString<int>(targetPosition[1]);
                moduleName += "_" + somethingToString<int>(targetPosition[2]);
            }
            else
            {
                cerr << "Position has got more than three coordinates" << std::endl;
                exit(EXIT_FAILURE);
            }

            moduleName += "_" + aut->getPSystemName();

            return moduleName;
        }
    }

    cerr << "No target module in rule " <<  rule.getName() << std::endl;
    exit(EXIT_FAILURE);

}

//Displayer
void PrismCommand::show()
{
    std::cout << "[" << this->identifier << "] " << this->predicate;
    std::cout << " -> " << std::endl;
    std::cout << "\t\t";
    std::cout << this->rate << " : " << std::endl;
    std::cout << "\t\t\t";

    for(std::vector<std::string>::iterator up = this->updates.begin();
    up != this->updates.end();
    ++up)
    {
        std::cout << *up << ";" << std::endl;
    }

}
